window.Gleam.initSearch([{"doc":"qcheck_gleeunit_utils","title":"qcheck_gleeunit_utils","content":"&lt;!-- TOC --&gt;&lt;a name=&quot;qcheck_gleeunit_utils&quot;&gt;&lt;/a&gt;\n# qcheck_gleeunit_utils\n\n[![Package Version](https://img.shields.io/hexpm/v/qcheck_gleeunit_utils)](https://hex.pm/packages/qcheck_gleeunit_utils)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/qcheck_gleeunit_utils/)\n\n\nThis package provides utility functions for working with Gleam&#39;s [gleeunit](https://github.com/lpil/gleeunit) test framework.\n\nWhile it may be more broadly useful, this library is mainly developed for internal use in the [qcheck](https://github.com/mooreryan/gleam_qcheck) library.  As such, no guarantees about API stability will be made until qcheck itself is more stable.\n\n## Contents\n\n&lt;!-- TOC start (generated with https://github.com/derlin/bitdowntoc) --&gt;\n\n- [qcheck_gleeunit_utils](#qcheck_gleeunit_utils)\n   * [Compilation target](#compilation-target)\n   * [Usage](#usage)\n      + [Run all tests in parallel](#run-all-tests-in-parallel)\n      + [Run tests with long timeouts](#run-tests-with-long-timeouts)\n      + [Run parallel test groups](#run-parallel-test-groups)\n         - [Gotchas](#gotchas)\n      + [Run in-order tests in a parallel context](#run-in-order-tests-in-a-parallel-context)\n   * [License](#license)\n\n&lt;!-- TOC end --&gt;\n\n&lt;!-- TOC --&gt;&lt;a name=&quot;compilation-target&quot;&gt;&lt;/a&gt;\n## Compilation target\n\nThe functions provided in this package only work properly on the Erlang target.  They will not manage your tests properly on the JavaScript target.\n\n&lt;!-- TOC --&gt;&lt;a name=&quot;usage&quot;&gt;&lt;/a&gt;\n## Usage\n\nThe following sections show some common use cases.  More info can be found in the docs.\n\n(Click the tiny arrows to expand the sections containing example code.)\n\n&lt;!-- TOC --&gt;&lt;a name=&quot;run-all-tests-in-parallel&quot;&gt;&lt;/a&gt;\n### Run all tests in parallel\n\nIf you want to keep things as simple as possible and simply run all your tests in parallel, you may replace the call to `gleeunit.main` with `run.run_gleeunit`.\n\n&lt;details&gt;\n\n&lt;summary style=&quot;color:#D900B8;&quot;&gt;\n  &lt;span style=&quot;text-decoration:underline;&quot;&gt;\n    Click to expand the example code!\n  &lt;/span&gt;\n&lt;/summary&gt;\n\n```gleam\nimport gleeunit/should\nimport qcheck_gleeunit_utils/run\n\npub fn main() {\n  run.run_gleeunit()\n}\n\npub fn example_1_should_pass__test() {\n  do_work()\n  should.equal(1, 1)\n}\n\npub fn example_2_should_fail__test() {\n  do_work()\n  should.equal(1, 2)\n}\n\npub fn example_3_should_pass__test() {\n  do_work()\n  should.equal(1, 1)\n}\n\npub fn example_4_should_fail__test() {\n  do_work()\n  should.equal(100, 200)\n}\n\nimport gleam/list\n\n// A small function simulating some CPU bound work.\nfn do_work() {\n  let _l = list.range(0, 10_000_000)\n\n  Nil\n}\n```\n\n&lt;/details&gt;\n\nBe aware that any individual test that takes longer than 5 seconds (the default EUnit timeout), will cause the whole test suite to come crashing down.\n\n&lt;!-- TOC --&gt;&lt;a name=&quot;run-tests-with-long-timeouts&quot;&gt;&lt;/a&gt;\n### Run tests with long timeouts\n\nHere is an example of a test that will timeout (...at least, it will timeout on my laptop).\n\n&lt;details&gt;\n\n&lt;summary style=&quot;color:#D900B8;&quot;&gt;\n  &lt;span style=&quot;text-decoration:underline;&quot;&gt;\n    Click to expand the example code!\n  &lt;/span&gt;\n&lt;/summary&gt;\n\n\n```gleam\nimport gleeunit\nimport gleeunit/should\n\npub fn main() {\n  gleeunit.main()\n}\n\npub fn hello__test() {\n  do_work(25)\n  should.equal(1, 1)\n}\n\nimport gleam/list\n\n// A small function simulating some CPU bound work.\nfn do_work(i) {\n  case i &gt;= 0 {\n    True -&gt; {\n      let _l = list.range(0, 10_000_000)\n      do_work(i - 1)\n    }\n    False -&gt; Nil\n  }\n}\n```\n\n&lt;/details&gt;\n\nIf you run that with `gleam test`, (and you&#39;re computer isn&#39;t too much faster than mine), then you should get a timeout error that looks sort of like this:\n\n```\nPending:\n  qcheck_gleeunit_utils_test.hello__test: module &#39;qcheck_gleeunit_utils_test&#39;\n    %% Unknown error: {timeout,\n\n... lots more stack trace ...\n```\nOkay, so that test is taking too long, so it crashes with the timeout error.\n\nWe can avoid that by using a test generating function along with the `test_spec.make` function.\n\n&lt;details&gt;\n\n&lt;summary style=&quot;color:#D900B8;&quot;&gt;\n  &lt;span style=&quot;text-decoration:underline;&quot;&gt;\n    Click to expand the example code!\n  &lt;/span&gt;\n&lt;/summary&gt;\n\n\n```gleam\nimport gleeunit\nimport gleeunit/should\n\n// Add this import statement.\nimport qcheck_gleeunit_utils/test_spec\n\npub fn main() {\n  gleeunit.main()\n}\n\n// Add a trailing `_` (underscore) character to the test name to specify that it\n// is a function for generating rather than a test itself.\npub fn hello__test_() {\n  // And use the `test_spec.make` function here.\n  test_spec.make(fn() {\n    do_work(25)\n    should.equal(1, 1)\n  })\n}\n\nimport gleam/list\n\n// A small function simulating some CPU bound work.\nfn do_work(i) {\n  case i &gt;= 0 {\n    True -&gt; {\n      let _l = list.range(0, 10_000_000)\n      do_work(i - 1)\n    }\n    False -&gt; Nil\n  }\n}\n```\n\nYou could also write the `hello__test_()` function with the `use` syntax if you prefer.\n\n```gleam\npub fn hello__test_() {\n  // And use the `test_spec.make` function here.\n  use &lt;- test_spec.make\n  do_work(25)\n  should.equal(1, 1)\n}\n```\n\n&lt;/details&gt;\n\nNow, run that with `gleam test` and you will get a passing test. (On my computer, it takes about 7 seconds.)\n\n*Note that this example can also be used with the `run.run_gleeunit` as opposed to the `gleeunit.main` if you want to parallelize all the tests.\n\n&lt;!-- TOC --&gt;&lt;a name=&quot;run-parallel-test-groups&quot;&gt;&lt;/a&gt;\n### Run parallel test groups\n\nSometimes, you may want only certain tests to be run in parallel, perhaps because you have some tests that interact with some external state or involve some other tricky setup or teardown that may interact poorly with other tests running in parallel.\n\nFor this, we use the `TestGroup`s, which are created with the `test_spec.run_in_parallel` and `test_spec.run_in_order` functions.  For this use case, we will use `test_spec.run_in_parallel` to create a `TestGroup` that will specify tests to be run in parallel.\n\n&lt;details&gt;\n\n&lt;summary style=&quot;color:#D900B8;&quot;&gt;\n  &lt;span style=&quot;text-decoration:underline;&quot;&gt;\n    Click to expand the example code!\n  &lt;/span&gt;\n&lt;/summary&gt;\n\n\n```gleam\nimport gleam/list\nimport gleeunit\nimport gleeunit/should\nimport qcheck_gleeunit_utils/test_spec\n\npub fn main() {\n  gleeunit.main()\n}\n\npub fn in_order_1__test() {\n  do_work(5)\n  should.equal(1, 1)\n}\n\npub fn in_order_2__test() {\n  do_work(5)\n  should.equal(1, 1)\n}\n\npub fn in_order_3__should_fail__test() {\n  do_work(5)\n  should.equal(1, 3)\n}\n\nfn in_parallel_1() {\n  do_work(5)\n  should.equal(1, 1)\n}\n\nfn in_parallel_2() {\n  do_work(5)\n  should.equal(1, 1)\n}\n\nfn in_parallel_3__should_fail() {\n  do_work(5)\n  should.equal(1, 11)\n}\n\npub fn in_parallel__test_() {\n  [in_parallel_1, in_parallel_2, in_parallel_3__should_fail]\n  // Note the use of `test_spec.make` here.\n  |&gt; list.map(test_spec.make)\n  |&gt; test_spec.run_in_parallel\n}\n\n// A small function simulating some CPU bound work.\nfn do_work(i) {\n  case i &gt;= 0 {\n    True -&gt; {\n      let _l = list.range(0, 10_000_000)\n      do_work(i - 1)\n    }\n    False -&gt; Nil\n  }\n}\n```\n\n&lt;/details&gt;\n\n&lt;!-- TOC --&gt;&lt;a name=&quot;gotchas&quot;&gt;&lt;/a&gt;\n#### Gotchas\n\nThere is a tricky gotcha that you need to be aware of with test groups.  It occurs when you give different timeouts for functions in a test group.\n\n&lt;details&gt;\n\n&lt;summary style=&quot;color:#D900B8;&quot;&gt;\n  &lt;span style=&quot;text-decoration:underline;&quot;&gt;\n    Click to expand the example code!\n  &lt;/span&gt;\n&lt;/summary&gt;\n\n```gleam\nimport gleam/list\nimport gleeunit\nimport gleeunit/should\nimport qcheck_gleeunit_utils/test_spec\n\npub fn main() {\n  gleeunit.main()\n}\n\n// All of these tests should fail, and they are all too long for the default\n// timeout.\n\nfn in_parallel_1__should_fail() {\n  do_work(25)\n  should.equal(1, 1)\n}\n\nfn in_parallel_2__should_fail() {\n  do_work(25)\n  should.equal(1, 10)\n}\n\nfn in_parallel_3__should_fail() {\n  do_work(25)\n  should.equal(1, 100)\n}\n\npub fn in_parallel__test_() {\n  [\n    // The `make` calls by default use a really long timeout.\n    test_spec.make(in_parallel_1__should_fail),\n    // For this one, we set a timeout of 1 second.\n    test_spec.make_with_timeout(1, in_parallel_2__should_fail),\n    test_spec.make(in_parallel_3__should_fail),\n  ]\n  |&gt; test_spec.run_in_parallel\n}\n\n// A small function simulating some CPU bound work.\nfn do_work(i) {\n  case i &gt;= 0 {\n    True -&gt; {\n      let _l = list.range(0, 10_000_000)\n      do_work(i - 1)\n    }\n    False -&gt; Nil\n  }\n}\n```\n\n&lt;/details&gt;\n\nIf you run that with `gleam test`, while you should see three test failures, you will only see the first test failure.  I suspect this is because the timeout error triggered by the second test silently brings down the whole test group.  \n\nIf you change the timeout to `100` seconds (or really any long enough value), and rerun `gleam test`, you will see all three tests failing as expected.\n\nTo summarize, be careful with test groups when one or more of the tests may timeout--it will silently crash the whole test group, swallowing any failures (or passes) for some (or all) of the tests.\n\n&lt;!-- TOC --&gt;&lt;a name=&quot;run-in-order-tests-in-a-parallel-context&quot;&gt;&lt;/a&gt;\n### Run in-order tests in a parallel context\n\nIf what you really want is *all* tests to be run in parallel, it can be a real drag to specify all your tests in parallel test groups.  So it is easy enough to use the `run.run_gleeunit` parallel helper.  However, what if you need some of the tests to be run in order?\n\nYou can do that by creating a `TestGroup` with the `test_spec.run_in_order` function.\n\n&lt;details&gt;\n\n&lt;summary style=&quot;color:#D900B8;&quot;&gt;\n  &lt;span style=&quot;text-decoration:underline;&quot;&gt;\n    Click to expand the example code!\n  &lt;/span&gt;\n&lt;/summary&gt;\n\n```gleam\nimport gleam/list\nimport gleeunit/should\nimport qcheck_gleeunit_utils/run\nimport qcheck_gleeunit_utils/test_spec\n\npub fn main() {\n  run.run_gleeunit()\n}\n\npub fn in_parallel_1__test() {\n  do_work(5)\n  should.equal(1, 1)\n}\n\npub fn in_parallel_2__test() {\n  do_work(5)\n  should.equal(1, 1)\n}\n\npub fn in_parallel_3__should_fail__test() {\n  do_work(5)\n  should.equal(1, 3)\n}\n\npub fn in_parallel_4__test() {\n  do_work(5)\n  should.equal(1, 1)\n}\n\npub fn in_parallel_5__test() {\n  do_work(5)\n  should.equal(1, 1)\n}\n\npub fn in_parallel_6__test() {\n  do_work(5)\n  should.equal(1, 1)\n}\n\nfn in_order_1() {\n  do_work(5)\n  should.equal(1, 1)\n}\n\nfn in_order_2() {\n  do_work(5)\n  should.equal(1, 1)\n}\n\nfn in_order_3__should_fail() {\n  do_work(5)\n  should.equal(1, 11)\n}\n\nfn in_order_4() {\n  do_work(5)\n  should.equal(1, 1)\n}\n\nfn in_order_5() {\n  do_work(5)\n  should.equal(1, 1)\n}\n\nfn in_order_6() {\n  do_work(5)\n  should.equal(1, 1)\n}\n\npub fn in_parallel__test_() {\n  [\n    in_order_1,\n    in_order_2,\n    in_order_3__should_fail,\n    in_order_4,\n    in_order_5,\n    in_order_6,\n  ]\n  |&gt; list.map(test_spec.make)\n  |&gt; test_spec.run_in_order\n}\n\n// A small function simulating some CPU bound work.\nfn do_work(i) {\n  case i &gt;= 0 {\n    True -&gt; {\n      let _l = list.range(0, 10_000_000)\n      do_work(i - 1)\n    }\n    False -&gt; Nil\n  }\n}\n```\n\n&lt;/details&gt;\n\nIf you run that with `gleam test`, you will see the first 6 test complete more or less all at once, then the following six tests complete one-by-one.\n\n&lt;!-- TOC --&gt;&lt;a name=&quot;license&quot;&gt;&lt;/a&gt;\n## License\n\n[![license MIT or Apache\n2.0](https://img.shields.io/badge/license-MIT%20or%20Apache%202.0-blue)](https://github.com/mooreryan/gleam_qcheck)\n\nCopyright (c) 2024 Ryan M. Moore\n\nLicensed under the Apache License, Version 2.0 or the MIT license, at your option. This program may not be copied, modified, or distributed except according to those terms.\n\nSee `licenses/gleeunit_license.txt` for the original gleeunit license.\n","url":"index.html"},{"doc":"qcheck_gleeunit_utils/run","title":"run_gleeunit","content":"pub fn run_gleeunit() -&gt; Nil\n Find and run all test functions for the current project using Erlang&#39;s EUnit\n test framework.\n\n Any Erlang or Gleam function in the `test` directory with a name ending in\n `_test` is considered a test function and will be run.\n\n When targeting Erlang, any Erlang or Gleam function in the `test` directory \n with a name ending in `_test_` is considered a test generating function that \n should return a representation of a set of tests to be run.  (See \n `test_spec.make`, for more information.)\n\n - If running on Erlang, tests will be run in parallel.\n - If running on JavaScript, tests will be run with Gleeunit&#39;s default runner.\n\n \nSynonyms:\nqcheck_gleeunit_utils/run.run_gleeunit\nqcheck_gleeunit_utils/run run_gleeunit","url":"qcheck_gleeunit_utils/run.html#run_gleeunit"},{"doc":"qcheck_gleeunit_utils/run","title":"qcheck_gleeunit_utils/run","content":" This module provides an alternative to the \n [gleeunit.main](https://hexdocs.pm/gleeunit/gleeunit.html#main) function \n that will run tests in parallel when targeting Erlang.  When targeting \n JavaScript, it will use Gleeunit&#39;s default runner.\n \n ","url":"qcheck_gleeunit_utils/run.html"},{"doc":"qcheck_gleeunit_utils/test_spec","title":"TestGroup","content":"pub opaque type TestGroup(a)\n `TestGroup(a)`represents a group of test specifications.\n \n \n\nSynonyms:\nqcheck_gleeunit_utils/test_spec.TestGroup\nqcheck_gleeunit_utils/test_spec TestGroup","url":"qcheck_gleeunit_utils/test_spec.html#TestGroup"},{"doc":"qcheck_gleeunit_utils/test_spec","title":"TestSpec","content":"pub opaque type TestSpec(a)\n `TestSpec(a)` represents a specification for a test.\n \n \n\nSynonyms:\nqcheck_gleeunit_utils/test_spec.TestSpec\nqcheck_gleeunit_utils/test_spec TestSpec","url":"qcheck_gleeunit_utils/test_spec.html#TestSpec"},{"doc":"qcheck_gleeunit_utils/test_spec","title":"make","content":"pub fn make(f: fn() -&gt; a) -&gt; TestSpec(a)\n `make(f)` creates a test specification that specifies how to run the\n function `f` with a very long timeout.\n \n While the function `f` can technically return a value of any type, it is\n likely that the return type will be `Nil`.  For example, when using \n functions from the `gleeunit/should` module.\n \n \n make(fn() {\n   should.equal(1 + 2, 3)\n })\n \n \n You may prefer the `use` syntax:\n \n \n use &lt;- make\n should.equal(1 + 2, 3)\n \n \n Named functions of the correct signature may also be used.\n \n \n fn addition_is_commutative() {\n   should.equal(1 + 2, 2 + 1)\n }\n \n // ... later inside some other function ...\n make(addition_is_commutative)\n \n \n \nSynonyms:\nqcheck_gleeunit_utils/test_spec.make\nqcheck_gleeunit_utils/test_spec make","url":"qcheck_gleeunit_utils/test_spec.html#make"},{"doc":"qcheck_gleeunit_utils/test_spec","title":"make_with_timeout","content":"pub fn make_with_timeout(\n  timeout: Int,\n  f: fn() -&gt; a,\n) -&gt; TestSpec(a)\n `make_with_timeout(timeout, f)` creates a test specification that specifies\n how to run the function `f` with a custom `timeout` in given in seconds.\n \n See [make](#make) for examples.\n \n \nSynonyms:\nqcheck_gleeunit_utils/test_spec.make_with_timeout\nqcheck_gleeunit_utils/test_spec make_with_timeout","url":"qcheck_gleeunit_utils/test_spec.html#make_with_timeout"},{"doc":"qcheck_gleeunit_utils/test_spec","title":"run_in_order","content":"pub fn run_in_order(\n  test_specs: List(TestSpec(a)),\n) -&gt; TestGroup(a)\n `run_in_order(test_specs)` creates a test group that specifies that the\n given `test_specs` should be run in order.\n \n See `run_in_parallel` for examples.\n \n \nSynonyms:\nqcheck_gleeunit_utils/test_spec.run_in_order\nqcheck_gleeunit_utils/test_spec run_in_order","url":"qcheck_gleeunit_utils/test_spec.html#run_in_order"},{"doc":"qcheck_gleeunit_utils/test_spec","title":"run_in_parallel","content":"pub fn run_in_parallel(\n  test_specs: List(TestSpec(a)),\n) -&gt; TestGroup(a)\n `run_in_parallel(test_specs)` creates a test group that specifies that the\n given `test_specs` should be run in parallel.\n \n The `run_in_parallel` function is generally used in the context of a \n [test generating function](https://www.erlang.org/doc/apps/eunit/chapter#writing-test-generating-functions).\n You write a function that returns a representation of the set of tests to be\n executed.  \n \n The names of these functions **must** end with `_test_` (note the trailing \n underscore).\n \n \n pub fn a_lengthy_nice_math_test_() {\n   [\n     make(fn() {\n       let result = some_lengthy_calculation(1, 2)\n       should.equal(1, result)\n     }),\n     make(fn() {\n       let result = another_lengthy_calculation(10, 20)\n       should.equal(100, result)\n     }),\n   ]\n   |&gt; run_in_parallel\n }\n \n \n \nSynonyms:\nqcheck_gleeunit_utils/test_spec.run_in_parallel\nqcheck_gleeunit_utils/test_spec run_in_parallel","url":"qcheck_gleeunit_utils/test_spec.html#run_in_parallel"},{"doc":"qcheck_gleeunit_utils/test_spec","title":"qcheck_gleeunit_utils/test_spec","content":" Utility functions for representing tests and test groups in Gleeunit, \n allowing users to control the timeout length of individual tests as well as \n to create groups of tests that will be run in parallel or in order.\n \n - [TestSpec](#TestSpec) values are created by [make](#make) and \n   [make_with_timeout](#make_with_timeout).\n - [TestGroup](#TestGroup) values are created by \n   [run_in_parallel](#run_in_parallel) and [run_in_order](#run_in_order).\n\n Both `TestSpec`s and `TestGroup`s represent tests as data, which, when \n targeting Erlang, will be executed by the test runner *if* they are \n returned by a test generating function (that is, a function whose name is \n prefixed by `_test_`).\n \n **Note:** The functions in this module will *NOT* work correclty on the \n JavaScript target.\n \n ","url":"qcheck_gleeunit_utils/test_spec.html"}]);